local _G = _G
local RW = _G.RW3
local expect = RW.Debug.expect
local DEBUG = RW.DEBUG

function RW:InitModLists()
	for i = 1, GetNumAddOns() do		
		-- RW3 parts must have "##X-RW3-AddOn" tag
		if GetAddOnMetadata(i, "X-RW3-AddOn") then
			local ID, name, notes, enabled, loadable, reason = GetAddOnInfo(i)
			
			if enabled and not IsAddOnLoaded(i) and IsAddOnLoadOnDemand(i) then
				local icon = GetAddOnMetadata(i, "X-RW3-Icon")
				local load_when = GetAddOnMetadata(i, "X-RW3-Load")
				local order = GetAddOnMetadata(i, "X-RW3-LoadOrder")

				RW.Debug:Print(9, ID, name, notes, enabled, loadable, reason, load_when)
				
				if load_when then					
					if DEBUG then
						expect(self.Load[load_when], "~=", nil)
					end
					tinsert(self.Load[load_when], {
						ID = ID, 
						Order = order or 10e5, 
						Name = name ,
						Icon = icon,
					} )
				end				
			end
		end
	end
	
	local function SortLoadOrder(a, b)
		return a.Order < b.Order
	end	
	
	for _, LoadKind in pairs(self.Load) do
		table.sort(LoadKind, SortLoadOrder)
	end
end

function RW:LoadAddOn(AddOn)
	if DEBUG then
		expect(AddOn, "typeof", "table")
	end
	RW.Debug:Print(5, "Loading Addon", AddOn.ID)
	local loaded, reason = LoadAddOn(AddOn.ID)
	if not loaded then
		print(("Failed to load %s: %s"):format(AddOn.Name, _G[("ADDON_%s"):format(reason)]))
	end
end