local name, addon_table = ...

local RW = LibStub("AceAddon-3.0"):NewAddon("Raid Watch 3", "AceEvent-3.0")
RW3 = RW

local Debug = addon_table.Debug
local expect = Debug.expect
RW.Debug = Debug

RW.DEBUG = true
local DEBUG = RW.DEBUG

local Callbacks = LibStub("CallbackHandler-1.0"):New(RW)
RW.Callbacks = Callbacks

RW.Load = {
	Enabled = {},
	CoreEnabled = {},
	CoreLoaded = {},
	CoreInitialized = {},
}

local Events = {}

function RW:OnInitialize()
	local defaults = {
		profile = {
			DebugLevel = 0,
			Color = {1, 0, 1, 1},
		}
	}
	
	self.db = LibStub("AceDB-3.0"):New("RaidWatch3DB", defaults, true)
	defaults = nil
	
	Debug.db = self.db
	
	self:InitModLists()
	
	self.Common:NamedTable(Events, "Loader_Events")
	self.RegisterCallback(Events, "Enabled")
	self.RegisterCallback(Events, "CoreLoaded")
	self.RegisterCallback(Events, "CoreInitialized")
	self.RegisterCallback(Events, "CoreEnabled")
end

function RW:OnEnable()
	Callbacks:Fire("Enabled")	
	
	self.LDB:AddClickHelpLine("LoadAndShowOptions", "Left Click to Load and Show options")
	
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	
	self.LDB:AddClickFunction("Left", "LoadConfig", function()
		RW:LoadAddOn({ ID = "RaidWatch3_Config", Name = "RaidWatch 3 [Config]" } )
		RW.LDB:RemoveClickFunction("Left", "LoadConfig")
		RW.LDB:RemoveClickHelpLine("LoadAndShowOptions")
		if RW.Config then
			RW.Config:Toggle()
		end
	end)
end

function RW:ZONE_CHANGED_NEW_AREA(event)
	
end

function RW:PLAYER_ENTERING_WORLD(event)
	self:ZONE_CHANGED_NEW_AREA("ZONE_CHANGED_NEW_AREA")
end

function Events:Enabled()
	for _, AddOn in pairs(RW.Load.Enabled) do
		RW:LoadAddOn(AddOn)
	end
end

function Events:CoreLoaded()
end

function Events:CoreInitialized()
end

function Events:CoreEnabled()
	for _, AddOn in pairs(RW.Load.CoreEnabled) do
		RW:LoadAddOn(AddOn)
	end
end

RW.Common = {}
function RW.Common:GUID_CID(guid)
	expect(guid, "typeof", "string")
	
	return tonumber(guid:sub(7, 10), 16)
end

local function tabletostring(t) return t._name end
function RW.Common:NamedTable(t, name)
	if DEBUG then
		expect(name, "typeof", "string")
	end
	t._name = name
	
	local newMeta = {}
	local oldmeta = getmetatable(t)
	if oldmeta then
		for k, v in pairs(oldmeta) do newMeta[k] = v end
	end
	newMeta.__tostring = tabletostring
	
	setmetatable(t, newMeta)
end
function RW.Common:IconTexture(icon)
	return ("Interface\\ICONS\\%s"):format(icon)
end

RW.Constants = setmetatable({}, {
	__index = function(self, k)
		if type(k) == "number" then
			for i, v in pairs(self) do
				if v == k then
					return i
				end
			end
		end
	end
})
function RW.Constants:Register(name, value)
	expect(name, "typeof", "string")
	expect(value, "typeof", "number")
	
	if self[name] then
		error(("Cannot register constant %q, it's allready registered."):format(name))
	end
	
	RW.Constants[name] = value
end

RW.Constants:Register("PARTY_TYPE_RAID", 1)
RW.Constants:Register("PARTY_TYPE_PARTY", 2)
RW.Constants:Register("PARTY_TYPE_BATTLEGROUND", 3)
RW.Constants:Register("PARTY_TYPE_ALONE", 4)