local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch 3")

local Debug = RW.Debug
local expect = Debug.expect
local Common = RW.Common
local Constants = RW.Constants

local plugin = RW:NewModule("Party", "AceEvent-3.0")
RW.Party = plugin
Debug:AddGlobal("Party", plugin)

local pName = UnitName("player")
local pClass = select(2, UnitClass("player"))


function plugin:OnInitialize()
	self.Type = "self"
	self.Group = {}
end

function plugin:OnEnable()
	self:RegisterEvent("RAID_ROSTER_UPDATE", "PartyUpdate")
	self:RegisterEvent("PARTY_MEMBERS_CHANGED", "PartyUpdate")
	
end

function plugin:PartyUpdate(event)
	local NewType = self:GetType()
	
	if NewType ~= self.Type and NewType ~= "self" then
		RW.Callbacks:Fire("PartyJoined", NewType)
	elseif NewType ~= self.Type and NewType == "self" then
		RW.Callbacks:Fire("PartyLeave", self.Type)
	end	
	
	self.Type = NewType
	
	if (self.Type == Constants.PARTY_TYPE_RAID or self.Type == Constants.PARTY_TYPE_BATTLEGROUND) and event == "RAID_ROSTER_UPDATE" then
	
		for i = 1, GetNumRaidMembers() do
			local name, rank, subgroup, level, class, fileName, zone, online, isDead, role, isML = GetRaidRosterInfo(i);
			if name and name ~= "Unknown" then
				if not self.Group[name] and name ~= pName then
					RW.Callbacks:Fire("PartyPlayerJoined", name)
				end				
			end		
			if name then
				self.Group[name] = self.Group[name] or {}
				self.Group[name].name = name
				self.Group[name].class = fileName
				self.Group[name].rank = rank
				self.Group[name].subgroup = subgroup
				self.Group[name].zone = zone
				self.Group[name].id = ("raid%d"):format(i)
				self.Group[name].updated = true
				RW.Callbacks:Fire("PartyPlayerUpdated", name)
			end
		end
		
		for i, player in pairs(self.Group) do
			if not player.updated then
				RW.Callbacks:Fire("PartyPlayerLeft", player.name)
				self.Group[i] = nil
			else
				player.updated = nil
			end
		end
		
		self.curParty = partyType
	
	elseif self.Type == Constants.PARTY_TYPE_PARTY and event == "PARTY_MEMBERS_CHANGED" then
	
		for i = 0, GetNumPartyMembers() do
			local id = i == 0 and "player" or ("party%d"):format(i)
			local name = UnitName(id)
			if name and name ~= "Unknown" then
				if not self.Group[name] and name ~= pName then
					RW.Callbacks:Fire("PartyPlayerJoined", name)
				end
			end
			if name then
				self.Group[name] = self.Group[name] or {}
				self.Group[name].name = name
				self.Group[name].class = select(2, UnitClass(id))
				self.Group[name].rank = UnitIsPartyLeader(id) and 2 or 0
				self.Group[name].id = id
				self.Group[name].updated = true
				RW.Callbacks:Fire("PartyPlayerUpdated", name)
			end
		end
		
		for i, player in pairs(self.Group) do
			if not player.updated then
				RW.Callbacks:Fire("PartyPlayerLeft", player.name)
				self.Group[i] = nil
			else
				player.updated = nil
			end
		end
		
		self.curParty = partyType
		
	elseif self.Type == Constants.PARTY_TYPE_ALONE then
		for i, player in pairs(self.Group) do
			self.Group[i] = nil
		end
		
		self.Group[pName] = self.Group[pName] or {}
		self.Group[pName].name = pName
		self.Group[pName].class = pClass
		self.Group[pName].rank = 2
		self.Group[pName].id = "player"
	end
end

--- Get the current party type
-- @return Current party type (raid, party, battleground)
function plugin:GetType()
	return ((select(2, IsInInstance()) == "pvp" or select(2, IsInInstance()) == "arena" and Constants.PARTY_TYPE_BATTLEGROUND) 
	or (GetNumRaidMembers() > 0 and Constants.PARTY_TYPE_RAID) 
	or (GetNumPartyMembers() > 0 and Constants.PARTY_TYPE_PARTY) 
	or Constants.PARTY_TYPE_ALONE) 
end

--- Get the current size of the party.
-- @return Current party size.
function plugin:GetSize() return  math.max(GetNumRaidMembers(), GetNumPartyMembers() + 1) end

--- Check if player is in current party by name
-- @param	name	string	Name of the player to check for
-- @return True if the player is in the party, else nil.
function plugin:InParty(name) if self.Group[name] then return true end end

--- Get an interator for the current party
-- @return An iterator to iterate over the current party.
function plugin:Iterator() return pairs(self.Group) end

--- Get the number of player currently alive in the party.
-- @return The number of alive people.
function plugin:GetNumAlive()
	local alive = 0
	for i, player in pairs(self.group) do
		alive = alive + (UnitIsDeadOrGhost(player.id) and 0 or 1)
	end
	return alive
end